package ru.dbotthepony.mc.otm.client.screen.panels.util

import net.minecraft.client.gui.screens.Screen
import ru.dbotthepony.mc.otm.client.render.RenderGravity
import ru.dbotthepony.mc.otm.client.screen.panels.Dock
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.client.screen.panels.slot.AbstractSlotPanel
import ru.dbotthepony.mc.otm.core.collect.filter
import ru.dbotthepony.mc.otm.core.collect.maybe
import kotlin.math.max

open class GridPanel<out S : Screen>(
	screen: S,
	parent: EditablePanel<*>?,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = 0f,
	height: Float = 0f,
	columns: Int,
	rows: Int
) : EditablePanel<S>(screen, parent, x, y, width, height) {
	var columns: Int = columns
		set(value) {
			if (field != value) {
				field = value
				invalidateLayout()
			}
		}

	var rows: Int = rows
		set(value) {
			if (field != value) {
				field = value
				invalidateLayout()
			}
		}

	var gravity: RenderGravity = RenderGravity.CENTER_CENTER
		set(value) {
			if (field != value) {
				field = value
				invalidateLayout()
			}
		}

	var layout: Layout = Layout.TOP_LEFT
		set(value) {
			if (field != value) {
				field = value
				invalidateLayout()
			}
		}

	enum class Layout {
		TOP_LEFT {
			override fun rows(last: Int): IntIterator {
				return (0 until last).iterator()
			}

			override fun columns(last: Int): IntIterator {
				return (0 until last).iterator()
			}
		},

		TOP_RIGHT {
			override fun rows(last: Int): IntIterator {
				return (0 until last).iterator()
			}

			override fun columns(last: Int): IntIterator {
				return (last - 1 downTo 0).iterator()
			}
		},
		BOTTOM_LEFT {
			override fun rows(last: Int): IntIterator {
				return (last - 1 downTo 0).iterator()
			}

			override fun columns(last: Int): IntIterator {
				return (0 until last).iterator()
			}
		},

		BOTTOM_RIGHT {
			override fun rows(last: Int): IntIterator {
				return (last - 1 downTo 0).iterator()
			}

			override fun columns(last: Int): IntIterator {
				return (last - 1 downTo 0).iterator()
			}
		};

		abstract fun rows(last: Int): IntIterator
		abstract fun columns(last: Int): IntIterator
	}

	override fun sizeToContents() {
		if (visibleChildren.isEmpty()) {
			// nothing to size against
			return
		}

		visibleChildren.forEach { it.sizeToContents() }

		var width = 0f
		var height = 0f

		val children = visibleChildren.iterator().filter { it.dock == Dock.NONE }

		for (row in layout.rows(rows)) {
			var maxHeight = 0f
			var rowWidth = 0f

			for (column in layout.columns(columns)) {
				val nextChild = children.maybe() ?: break
				rowWidth += nextChild.width + nextChild.dockMargin.horizontal
				maxHeight = max(maxHeight, nextChild.height + nextChild.dockMargin.vertical)
			}

			height += maxHeight
			width = max(width, rowWidth)
		}

		this.width = width
		this.height = height
	}

	override fun performLayout() {
		super.performLayout()
		var children = visibleChildren.iterator().filter { it.dock == Dock.NONE }

		var totalWidth = 0f
		var totalHeight = 0f

		for (row in layout.rows(rows)) {
			var maxHeight = 0f
			var width = 0f

			for (column in layout.columns(columns)) {
				val child = children.maybe() ?: break
				width += child.dockedWidth
				maxHeight = maxHeight.coerceAtLeast(child.dockedHeight)
			}

			totalWidth = totalWidth.coerceAtLeast(width)
			totalHeight += maxHeight
		}

		val alignX = gravity.repositionX(width, totalWidth)
		val alignY = gravity.repositionY(height, totalHeight)
		children = visibleChildren.iterator().filter { it.dock == Dock.NONE }

		totalWidth = 0f
		totalHeight = 0f

		for (row in layout.rows(rows)) {
			var maxHeight = 0f
			var width = 0f

			for (column in layout.columns(columns)) {
				val child = children.maybe() ?: break
				child.x = alignX + width
				child.y = alignY + totalHeight
				width += child.dockedWidth
				maxHeight = maxHeight.coerceAtLeast(child.dockedHeight)
			}

			totalWidth = totalWidth.coerceAtLeast(width)
			totalHeight += maxHeight
		}
	}

	companion object {
		fun <S : Screen> slots(screen: S, parent: EditablePanel<*>?, columns: Int, rows: Int): GridPanel<S> {
			return GridPanel(screen, parent, width = columns * AbstractSlotPanel.SIZE, height = rows * AbstractSlotPanel.SIZE, rows = rows, columns = columns)
		}
	}
}
