package ru.dbotthepony.mc.otm.menu.decorative

import net.minecraft.tags.ItemTags
import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.*
import net.minecraft.world.item.crafting.RecipeHolder
import net.minecraft.world.level.material.Fluids
import net.neoforged.neoforge.capabilities.Capabilities
import net.neoforged.neoforge.fluids.FluidStack
import net.neoforged.neoforge.fluids.capability.IFluidHandler
import ru.dbotthepony.kommons.util.Listenable
import ru.dbotthepony.kommons.util.ListenableDelegate
import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.kommons.util.setValue
import ru.dbotthepony.mc.otm.block.entity.decorative.PainterBlockEntity
import ru.dbotthepony.mc.otm.player.matteryPlayer
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.mc.otm.core.addAll
import ru.dbotthepony.mc.otm.core.collect.SupplierMap
import ru.dbotthepony.mc.otm.core.collect.filter
import ru.dbotthepony.mc.otm.core.collect.maybe
import ru.dbotthepony.mc.otm.core.isNotEmpty
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.input.BooleanInputWithFeedback
import ru.dbotthepony.mc.otm.menu.input.ItemConfigPlayerInput
import ru.dbotthepony.mc.otm.network.StreamCodecs
import ru.dbotthepony.mc.otm.network.nullable
import ru.dbotthepony.mc.otm.recipe.AbstractPainterRecipe
import ru.dbotthepony.mc.otm.registry.game.MMenus
import ru.dbotthepony.mc.otm.registry.game.MRecipes
import java.util.*
import java.util.function.IntSupplier
import kotlin.collections.ArrayList

class PainterMenu(
	containerId: Int, inventory: Inventory, tile: PainterBlockEntity? = null
) : MatteryMenu(MMenus.PAINTER, containerId, inventory, tile) {
	val dyeStored = (DyeColor.entries.toMutableList<DyeColor?>().also { it.add(0, null) }).associateWith { dye ->
		mSynchronizer.computedInt(IntSupplier { tile?.dyeStored(dye) ?: 0 }).also { it.addListener(Runnable { rescan() }) }
	}

	val dyeStoredDirect = SupplierMap(dyeStored)
	val itemConfig = ItemConfigPlayerInput(this, tile?.config)

	val inputContainer = SlottedContainer.simple(1, ::rescan)
	val outputContainer = SlottedContainer.simple(1)
	private var lastRecipe: RecipeHolder<out AbstractPainterRecipe>? = null
	var selectedRecipe by mSynchronizer.add(ListenableDelegate.Box(null), StreamCodecs.RESOURCE_LOCATION.nullable()).also { it.addListener(Runnable { rescan() }) }

	val isBulk = BooleanInputWithFeedback(this, tile?.let { it::isBulk })

	val selectRecipe = PlayerInput(StreamCodecs.RESOURCE_LOCATION) {
		selectedRecipe = it
	}

	val inputSlot = object : MatteryMenuSlot(inputContainer, 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			if (!itemStack.isEmpty && itemStack.`is`(ItemTags.DYEABLE)) {
				return super.mayPlace(itemStack)
			}

			return super.mayPlace(itemStack) && inventory.player.level().recipeManager.byType(MRecipes.PAINTER).any { it.value.matches(itemStack) }
		}
	}

	val outputSlot = object : MatteryMenuSlot(outputContainer, 0) {
		override fun tryRemove(p_150642_: Int, p_150643_: Int, p_150644_: Player): Optional<ItemStack> {
			rescan()
			return super.tryRemove(p_150642_, p_150643_, p_150644_)
		}

		override fun onTake(player: Player, itemStack: ItemStack) {
			if (itemStack.isNotEmpty) {
				lastRecipe?.value?.dyes?.let { tile?.takeDyes(it) }

				if (isBulk.value) {
					val found = player.matteryPlayer.inventoryAndExopack
						.slotIterator()
						//.filter { !it.isForbiddenForAutomation && ItemStack.isSameItemSameComponents(it.item, inputSlot.item) }
						.filter { ItemStack.isSameItemSameComponents(it.item, inputSlot.item) }
						.maybe()

					if (found != null) {
						found.remove(1)
						rescan()
					} else {
						inputContainer.removeItem(0, 1)
					}
				} else {
					inputContainer.removeItem(0, 1)
				}
			}

			super.onTake(player, itemStack)
		}

		override fun mayPlace(itemStack: ItemStack): Boolean {
			return false
		}
	}

	val dyeSlot = object : MatteryMenuSlot(tile?.dyeInput ?: SlottedContainer.simple(1), 0) {
		override fun mayPlace(itemStack: ItemStack): Boolean {
			return super.mayPlace(itemStack) && (
				itemStack.getCapability(Capabilities.FluidHandler.ITEM)
					?.drain(FluidStack(Fluids.WATER, PainterBlockEntity.MAX_WATER_STORAGE - dyeStoredDirect[null]!!), IFluidHandler.FluidAction.SIMULATE)?.isNotEmpty
					?: false
			|| DyeColor.getColor(itemStack) != null)
		}
	}

	init {
		addSlot(outputSlot)
		addSlot(dyeSlot)

		addStorageSlot(dyeSlot)
		addStorageSlot(inputSlot)
		mapQuickMoveToInventory(outputSlot)
		mapQuickMoveToInventory(dyeSlot)

		addInventorySlots()
	}

	override fun removed(p_38940_: Player) {
		super.removed(p_38940_)
		clearContainer(p_38940_, inputContainer)
	}

	val listeners = Listenable.Void()
	val possibleRecipes = ArrayList<RecipeHolder<out AbstractPainterRecipe>>()

	private fun rescan() {
		possibleRecipes.clear()
		possibleRecipes.addAll(inventory.player.level().recipeManager.byType(MRecipes.PAINTER).iterator().filter { it.value.matches(inputContainer[0]) })

		listeners.run()
		if (tile !is PainterBlockEntity) return

		if (inputContainer.isEmpty || selectedRecipe == null) {
			outputContainer.clearContent()
		} else {
			val recipe = inventory.player.level().recipeManager.byKey(selectedRecipe!!).get() as RecipeHolder<AbstractPainterRecipe>?

			if (recipe == null || !recipe.value.canCraft(dyeStoredDirect) || !recipe.value.matches(inputContainer, inventory.player.level())) {
				outputContainer.clearContent()
			} else {
				outputContainer[0] = recipe.value.assemble(inputContainer, inventory.player.level().registryAccess())
				lastRecipe = recipe
			}
		}
	}
}
