package ru.dbotthepony.mc.otm.menu.matter

import com.google.common.collect.ImmutableList
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.block.entity.matter.MatterBottlerBlockEntity
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.matter.canExtractMatter
import ru.dbotthepony.mc.otm.capability.matter.canReceiveMatter
import ru.dbotthepony.mc.otm.container.CombinedContainer
import ru.dbotthepony.mc.otm.container.slotted.SlottedContainer
import ru.dbotthepony.mc.otm.menu.input.BooleanInputWithFeedback
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.menu.MatteryPoweredMenu
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.UserFilteredMenuSlot
import ru.dbotthepony.mc.otm.menu.makeSlots
import ru.dbotthepony.mc.otm.menu.makeUpgradeSlots
import ru.dbotthepony.mc.otm.menu.widget.ProfiledLevelGaugeWidget
import ru.dbotthepony.mc.otm.registry.game.MMenus

class MatterBottlerMenu(
	p_38852_: Int,
	inventory: Inventory,
	tile: MatterBottlerBlockEntity? = null
) : MatteryPoweredMenu(MMenus.MATTER_BOTTLER, p_38852_, inventory, tile) {
	val workFlow = BooleanInputWithFeedback(this)
	val progressWidget = ProgressGaugeWidget(this)
	val matterWidget = ProfiledLevelGaugeWidget(this, tile?.matter, LevelGaugeWidget(this, tile?.matter))

	val storageSlots: ImmutableList<UserFilteredMenuSlot> = makeSlots(CombinedContainer(tile?.bottling ?: SlottedContainer.filtered(3), tile?.unbottling ?: SlottedContainer.filtered(3))) { it, index ->
		object : UserFilteredMenuSlot(it, index) {
			override fun mayPlace(itemStack: ItemStack): Boolean {
				val cap = itemStack.getCapability(MatteryCapability.MATTER_ITEM) ?: return false

				if (workFlow.value) {
					return index < 3 && cap.canReceiveMatter
				} else {
					return index >= 3 && cap.canExtractMatter
				}
			}
		}.also(this::addStorageSlot)
	}

	val profiledEnergy = ProfiledLevelGaugeWidget(this, tile?.energy, energyWidget)
	val upgrades = makeUpgradeSlots(3, tile?.upgrades)

	init {
		if (tile != null) {
			progressWidget.with(tile::workProgress)
			workFlow.with(tile::isBottling)
		}

		addInventorySlots()
	}
}
